import Flutter
import UIKit
import KeychainAccess

public class SwiftFlutterKeychainaccessPlugin: NSObject, FlutterPlugin {
  public static func register(with registrar: FlutterPluginRegistrar) {
    let channel = FlutterMethodChannel(name: "flutter_keychainaccess", binaryMessenger: registrar.messenger())
    let instance = SwiftFlutterKeychainaccessPlugin()
    registrar.addMethodCallDelegate(instance, channel: channel)
  }

  public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
    guard
        let args = call.arguments as? [String: String],
        let service = args["service"]
    else {
        return
    }

    let keychain = Keychain(service: service).synchronizable(true)
    if call.method == "write",
        let key = args["key"],
        let value = args["value"] {
        keychain[string: key] = value
    }
    if call.method == "read",
        let key = args["key"] {
        result(keychain[string: key])
    }
    if call.method == "allItems" {
        let items = keychain.allItems()
        let dict = items.reduce(into: [String: String]()) { dict, item in
            dict[item["key"]] = item["value"] as! String
        }
        result(dict)
    }
  }
}
